@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2016 PLT Design Inc.
+  Copyright (c) 2004-2017 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -31,6 +31,7 @@
 #include "schminc.h"
 #include "schmach.h"
 #include "schexpobs.h"
+#include "schrktio.h"
 #ifdef MZ_USE_FUTURES
 # include "future.h"
 #endif
@@ -177,15 +178,6 @@ void os_platform_init() {
     setrlimit(RLIMIT_STACK, &rl);
   }
 #endif
-#ifdef UNIX_LIMIT_FDSET_SIZE
-  struct rlimit rl;
-
-  getrlimit(RLIMIT_NOFILE, &rl);
-  if (rl.rlim_cur > FD_SETSIZE) {
-    rl.rlim_cur = FD_SETSIZE;
-    setrlimit(RLIMIT_NOFILE, &rl);
-  }
-#endif
 }
 
 Scheme_Env *scheme_restart_instance() {
@@ -299,10 +291,7 @@ Scheme_Env *scheme_engine_instance_init()
   scheme_init_logging_once();
 
   scheme_init_compenv_symbol();
-
-#if defined(MZ_PLACES_WAITPID)
-  scheme_places_start_child_signal_handler();
-#endif
+  scheme_init_param_symbol();
 
 #if defined(MZ_PRECISE_GC) && defined(MZ_USE_PLACES)
   GC_switch_out_master_gc();
@@ -340,6 +329,7 @@ static void init_unsafe(Scheme_Env *env)
   scheme_init_unsafe_hash(unsafe_env);
   scheme_init_unsafe_vector(unsafe_env);
   scheme_init_unsafe_fun(unsafe_env);
+  scheme_init_unsafe_thread(unsafe_env);
 
   scheme_init_extfl_unsafe_number(unsafe_env);
   scheme_init_extfl_unsafe_numarith(unsafe_env);
@@ -544,6 +534,7 @@ static Scheme_Env *place_instance_init(void *stack_base, int initial_main_os_thr
   scheme_init_error_escape_proc(NULL);
   scheme_init_print_buffers_places();
   scheme_init_thread_places();
+  scheme_init_fd_semaphores();
   scheme_init_string_places();
   scheme_init_logger();
   scheme_init_eval_places();
@@ -551,8 +542,6 @@ static Scheme_Env *place_instance_init(void *stack_base, int initial_main_os_thr
   scheme_init_regexp_places();
   scheme_init_sema_places();
   scheme_init_gmp_places();
-  scheme_init_kqueue();
-  scheme_alloc_global_fdset();
 #ifndef DONT_USE_FOREIGN
   scheme_init_foreign_places();
 #endif
@@ -571,9 +560,8 @@ static Scheme_Env *place_instance_init(void *stack_base, int initial_main_os_thr
   scheme_init_error_config();
 
 /* BEGIN PRIMITIVE MODULES */
-#ifndef NO_TCP_SUPPORT
+  scheme_init_linklet(env);
   scheme_init_network(env);
-#endif
   scheme_init_paramz(env);
   scheme_init_expand_observe(env);
   scheme_init_place(env);
@@ -622,10 +610,13 @@ Scheme_Env *scheme_place_instance_init(void *stack_base, struct NewGC *parent_gc
   int *signal_fd;
   GC_construct_child_gc(parent_gc, memory_limit);
 # endif
+  scheme_rktio = rktio_init();
   env = place_instance_init(stack_base, 0);
 # if defined(MZ_PRECISE_GC)
-  signal_fd = scheme_get_signal_handle();
-  GC_set_put_external_event_fd(signal_fd);
+  if (scheme_rktio) {
+    signal_fd = scheme_get_signal_handle();
+    GC_set_put_external_event_fd(signal_fd);
+  }
 # endif
   scheme_set_can_break(1);
   return env; 
@@ -656,10 +647,8 @@ void scheme_place_instance_destroy(int force)
   else
     scheme_run_atexit_closers_on_all(force_more_closed_after);
 
-#ifdef WINDOWS_PROCESSES
-  scheme_release_process_job_object();
-#endif
-
+  scheme_release_fd_semaphores();
+  
   scheme_release_file_descriptor();
 
   scheme_end_futures_per_place();
@@ -671,9 +660,7 @@ void scheme_place_instance_destroy(int force)
   GC_destruct_child_gc();
 #endif
   scheme_free_all_code();
-  scheme_free_ghbn_data();
-  scheme_release_kqueue();
-  scheme_release_inotify();
+  rktio_destroy(scheme_rktio);
 }
 
 static void make_kernel_env(void)
@@ -1597,6 +1584,7 @@ void scheme_shadow(Scheme_Env *env, Scheme_Object *n, Scheme_Object *val, int as
   if (!id) {
     if (env->module)
       return;
+    scheme_prepare_env_stx_context(env);
     id = scheme_datum_to_syntax(n, scheme_false, scheme_false, 0, 0);
     id = scheme_stx_add_module_context(id, env->stx_context);
   }
@@ -1610,7 +1598,9 @@ void scheme_shadow(Scheme_Env *env, Scheme_Object *n, Scheme_Object *val, int as
 
   scheme_add_module_binding(id, scheme_env_phase(env),
                             (env->module
-                             ? env->module->self_modidx
+                             ? (env->link_midx
+                                ? env->link_midx
+                                : env->module->self_modidx)
                              : scheme_false),
                             ((env->module && env->module->prefix)
                              ? env->module->prefix->src_insp_desc
@@ -1621,7 +1611,7 @@ void scheme_shadow(Scheme_Env *env, Scheme_Object *n, Scheme_Object *val, int as
   /* If the binding is a rename transformer, also install 
      a mapping */
   if (scheme_is_binding_rename_transformer(val))
-    scheme_add_binding_copy(id, scheme_rename_transformer_id(val), scheme_env_phase(env));
+    scheme_add_binding_copy(id, scheme_rename_transformer_id(val, NULL), scheme_env_phase(env));
 }
 
 static void install_one_binding_name(Scheme_Hash_Table *bt, Scheme_Object *name, Scheme_Object *id, Scheme_Env *benv)
@@ -1864,6 +1854,7 @@ namespace_identifier(int argc, Scheme_Object *argv[])
   obj = argv[0];
   obj = scheme_datum_to_syntax(obj, scheme_false, scheme_false, 1, 0);
 
+  scheme_prepare_env_stx_context(genv);
   obj = scheme_stx_add_module_context(obj, genv->stx_context);
 
   return obj;
@@ -2117,8 +2108,9 @@ static Scheme_Object *do_variable_namespace(const char *who, int tl, int argc, S
     return env->access_insp;
   } else if (tl) {
     /* return env directly; need to set up  */
-    if (!env->phase && env->module)
+    if (!env->mod_phase && env->module)
       scheme_prep_namespace_rename(env);
+    env->interactive_bindings = 1;
   } else {
     /* new namespace: */
     Scheme_Env *new_env;
@@ -2147,7 +2139,7 @@ static Scheme_Object *variable_phase(int argc, Scheme_Object *argv[])
 
 static Scheme_Object *variable_base_phase(int argc, Scheme_Object *argv[])
 {
-  return do_variable_namespace("variable-reference->phase", 3, argc, argv);
+  return do_variable_namespace("variable-reference->base-phase", 3, argc, argv);
 }
 
 static Scheme_Object *variable_inspector(int argc, Scheme_Object *argv[])
@@ -2217,9 +2209,17 @@ static Scheme_Object *variable_modidx(int argc, Scheme_Object *argv[])
     scheme_wrong_contract("variable-reference->module-path-index", "variable-reference?", 0, argc, argv);
 
   if (env->module) {
-    if (!env->link_midx) 
-      return env->module->self_modidx;
-    else
+    if (!env->link_midx) {
+      if (env->module->self_modidx
+          && SCHEME_TRUEP(((Scheme_Modidx *)env->module->self_modidx)->path))
+        return env->module->self_modidx;
+      else
+        return scheme_make_modidx(scheme_make_pair(scheme_intern_symbol("quote"),
+                                                   scheme_make_pair(scheme_resolved_module_path_value(env->module->modname),
+                                                                    scheme_null)),
+                                  scheme_false,
+                                  scheme_false);
+    } else
       return env->link_midx;
   } else
     return scheme_false;
@@ -2256,9 +2256,7 @@ now_transforming_with_lifts(int argc, Scheme_Object *argv[])
 {
   Scheme_Comp_Env *env = scheme_current_thread->current_local_env;
 
-  while (env && !env->lifts) {
-    env = env->next;
-  }
+  env = scheme_get_env_for_lifts(env);
 
   if (env)
     if (SCHEME_FALSEP(SCHEME_VEC_ELS(env->lifts)[0]))
@@ -2360,7 +2358,7 @@ do_local_exp_time_value(const char *name, int argc, Scheme_Object *argv[], int r
     
     v = SCHEME_PTR_VAL(v);
     if (scheme_is_rename_transformer(v)) {
-      sym = scheme_transfer_srcloc(scheme_rename_transformer_id(v), sym);
+      sym = scheme_transfer_srcloc(scheme_rename_transformer_id(v, NULL), sym);
       renamed = 1;
       menv = NULL;
       SCHEME_USE_FUEL(1);
@@ -2983,7 +2981,7 @@ rename_transformer_target(int argc, Scheme_Object *argv[])
   if (!scheme_is_rename_transformer(argv[0]))
     scheme_wrong_contract("rename-transformer-target", "rename-transformer?", 0, argc, argv);
 
-  return scheme_rename_transformer_id(argv[0]);
+  return scheme_rename_transformer_id(argv[0], NULL);
 }
 
 static Scheme_Object *
